// -----------------------------------------------------------------------------
//
// Copyright 2012(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
// FILE NAME : fpga.v
// MODULE NAME : fpga
// AUTHOR : ACostina
// AUTHORS EMAIL :adrian.costina@analog.com
// -----------------------------------------------------------------------------
// SVN REVISION: $WCREV$
// -----------------------------------------------------------------------------
// KEYWORDS : BeMicro_SDK
// -----------------------------------------------------------------------------
// PURPOSE : Top file for NiosCPU project. This project defines the controller
// hardware used for evaluating a number of parts from the Analog Devices
// System Demonstration Family.
// There are 2 cores for communication via I2C.
// There are 4 cores for communication via SPI. Two cores can be used
// simulatenously, as long as they have different CPHA/CPOL combination. 
// en_spi_0 activates the communication via the SPI ports,and en_spi_1 activates
// the communication via the SPORT ports.
// The SPI cores are named as follows: SPI_*CPOL*_P*CPHA*. 
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Reset Strategy      :
// Clock Domains       : ext_clk 50 MHz External clock
//                       sys_clk 100MHz deriver with a PLL, used as main
//                       system clock
// Critical Timing     :
// Test Features       :
// Asynchronous I/F    :
// Instantiations      :
// Synthesizable (y/n) : yes
// Target Device       : BeMicro SDK, EP4CE22F17C7
// Other               :
// ----------------------------------------------------------------------------

module fpga
    (
    input           ext_clk,        // external clock, 50 MHz
    output  [7:0]   led,            // 8 single LEDs
    output          epcs_dclk,      // EPCS device
    output          epcs_asdi,
    output          epcs_cs_n,
    input           epcs_data,
    input           exp_pres,       // expansion board is present
    input           exp_rst_n,      // reset switch on expansion board

               // bemicro signal       SDP pin, signal
    input           p1,             //  99, par_int
    input           p2,             //  83, spi_miso
    inout           p3,             // 107 + 91, par_d10 + sport_dr0
    inout           p4,             // 106 + 92, par_d8 + sport_rsclk
    inout           p5,             // 105, par_d6
    inout           p6,             // 103, par_d4
    inout           p7,             // 102, par_d2
    inout           p8,             // 101, par_d0
    inout           p9,             // 100, par_wr
    inout           p10,            // 110 + 89, par_d15 + sport_tfs
    inout           p11,            //  97, par_a2
    inout           p12,            //  96, par_a0
    inout           p13,            //  78, gpio1
    inout           p14,            //  94, par_clk
    inout           p15,            //  85, spi_sel_a
    inout           p16,            //  84, spi_mosi
    inout           p17,            //  82, spi_clk
    inout           p18,            // 108 + 90, par_d12 + sport_rfs
    inout           p19,            //  77, gpio3
    inout           p20,            //  79, scl_0
    inout           p21,            //  95, par_fs2
    inout           p22,            //  80, sda_0
    inout           p23,            //  76, gpio5
    inout           p24,            //  74, gpio7
    inout           p25,            //  73, tmr_b
    inout           p26,            //  72, tmr_d
    inout           p27,            //  43, gpio0
    inout           p28,            //  62, uart_tx
    inout           p29,            //  13 + 87, par_d13 + sport_tsclk
    input           p35,            //  35, sport_int
    input           p36,            //  59, uart_rx
    inout           p37,            //  47, gpio6
    inout           p38,            //  15 + 31, par_d9 + sport_dr1
    inout           p39,            //  16, par_d7
    inout           p40,            //  18, par_d5
    inout           p41,            //  19, par_d3
    inout           p42,            //  20, par_d1,
    inout           p43,            //  21, par_rd
    inout           p44,            //  22, par_cs
    inout           p45,            //  48, tmr_a
    inout           p46,            //  25, par_a1
    inout           p47,            //  26, par_fs3
    inout           p48,            //  27, par_fs1
    inout           p49,            //  12 + 88, par_d14 + sport_dt0
    inout           p50,            //  37, spi_sel_b
    inout           p51,            //  38, spi_sel_c
    inout           p52,            //  41, sda_1
    inout           p53,            //  42, scl_1
    inout           p54,            //  71, n/c
    inout           p55,            //  44, gpio2
    inout           p56,            //  45, gpio4
    inout           p57,            //  14 + 32, par_d11 + sport_dt1
    inout           p58,            //  24, par_a3
    inout           p59,            //  49, tmr_c,

    output          p60             //  reset request, driving U2.2 on interposer board
    );

//----------------------------------------------------------------------
// local wires
//----------------------------------------------------------------------

wire    [18:0]  addr;               // byte address
wire    [1:0]   ram_be_n;           // byte enable bus
wire    [7:0]   led_port;           // high-active signals

wire    [7:0]   ctrl;               // global control
wire    [2:0]   adc_data_ready;     // used for some ADCs: adc_data_ready[0]= SPI_MISO;adc_data_ready[1]=sport_dr0

wire            en_spi_0;           // enable output buffers for SPI
wire            cpol_0;             // invert the SPI clock
wire            clk_spi_0;          // clock signal for SPI interface
wire            phase_spi_0;        // phase selection for SPI interface (CPHA)
wire            en_spi_1;           // enable output buffers for SPORT
wire            cpol_1;             // invert the SPI via SPORT clock
wire            clk_spi_1;          // clock signal for SPORT interface
wire            phase_spi_1;        // phase selection for SPORT interface (CPHA)

wire            miso_0_0;           //CPOL 0; CPHA 0
wire            mosi_0_0;
wire            sclk_0_0;
wire    [2:0]   ss_n_0_0;           // if using SPORT, slave 0 TFS, slave 1 RFS, slave 2 activates both RFS and TFS
                                    // for SPI, 3 slaves are available

wire            miso_0_1;           //CPOL 0; CPHA 1
wire            mosi_0_1;
wire            sclk_0_1;
wire    [2:0]   ss_n_0_1;           // if using SPORT, slave 0 TFS, slave 1 RFS, slave 2 activates both RFS and TFS
                                    // for SPI, 3 slaves are available


wire            miso_1_0;           //CPOL 1; CPHA 0
wire            mosi_1_0;
wire            sclk_1_0;
wire    [2:0]   ss_n_1_0;           // if using SPORT, slave 0 TFS, slave 1 RFS, slave 2 activates both RFS and TFS
                                    // for SPI, 3 slaves are available

wire            miso_1_1;           //CPOL 1; CPHA 1
wire            mosi_1_1;
wire            sclk_1_1;
wire    [2:0]   ss_n_1_1;           // if using SPORT, slave 0 TFS, slave 1 RFS, slave 2 activates both RFS and TFS
                                    // for SPI, 3 slaves are available
wire    [2:0]   sport_ss_n;         // variable used for selecting the sport pins
wire            sport_dt0;          // variable used sending data
wire    [2:0]   spi_active;
wire    [2:0]   sport_active;
//----------------------------------------------------------------------
// SOPC system
//----------------------------------------------------------------------

uC uC1
    (
        .ext_clk(ext_clk),
        .reset_n(1'b1),
        .sys_clk(),
        .locked_from_the_main_pll(),
        .phasedone_from_the_main_pll(),

        .dclk_from_the_epcs_flash_controller(epcs_dclk),
        .sce_from_the_epcs_flash_controller(epcs_cs_n),
        .sdo_from_the_epcs_flash_controller(epcs_asdi),
        .data0_to_the_epcs_flash_controller(epcs_data),

        .out_port_from_the_led(led_port),
        .out_port_from_the_ctrl(ctrl),

        .scl_pad_io_to_and_from_the_opencores_i2c_0 (p20),
        .scl_pad_io_to_and_from_the_opencores_i2c_1 (p53),
        .sda_pad_io_to_and_from_the_opencores_i2c_0 (p22),
        .sda_pad_io_to_and_from_the_opencores_i2c_1 (p52),


        .MISO_to_the_spi_0_p0(miso_0_0),
        .MOSI_from_the_spi_0_p0(mosi_0_0),
        .SCLK_from_the_spi_0_p0(sclk_0_0),
        .SS_n_from_the_spi_0_p0(ss_n_0_0),

        .MISO_to_the_spi_0_p1(miso_0_1),
        .MOSI_from_the_spi_0_p1(mosi_0_1),
        .SCLK_from_the_spi_0_p1(sclk_0_1),
        .SS_n_from_the_spi_0_p1(ss_n_0_1),

        .MISO_to_the_spi_1_p0(miso_1_0),
        .MOSI_from_the_spi_1_p0(mosi_1_0),
        .SCLK_from_the_spi_1_p0(sclk_1_0),
        .SS_n_from_the_spi_1_p0(ss_n_1_0),

        .MISO_to_the_spi_1_p1(miso_1_1),
        .MOSI_from_the_spi_1_p1(mosi_1_1),
        .SCLK_from_the_spi_1_p1(sclk_1_1),
        .SS_n_from_the_spi_1_p1(ss_n_1_1),

        .bidir_port_to_and_from_the_gpio({p24, p37, p23, p56, p19, p55, p13, p27}), // GPIO pins
        .in_port_to_the_adc_data_ready              (adc_data_ready)
    );

// assign control bits
assign en_spi_0     = ctrl[0];
assign cpol_0       = ctrl[1];
assign phase_spi_0  = ctrl[2];
assign en_spi_1     = ctrl[4];
assign cpol_1       = ctrl[5];
assign phase_spi_1  = ctrl[6];

//select spi and sport interfaces
assign spi_active   = {en_spi_0,cpol_0,phase_spi_0};
assign sport_active = {en_spi_1,cpol_1,phase_spi_1};

// SPI 0
assign clk_spi_0        = phase_spi_0 ? (cpol_0 ? sclk_1_1 : sclk_0_1) : (cpol_0 ? sclk_1_0 : sclk_0_0);
assign p16              = phase_spi_0 ? (en_spi_0 ? (cpol_0 ? mosi_1_1 : mosi_0_1): 1'bz) : (en_spi_0 ? (cpol_0 ? mosi_1_0 : mosi_0_0) : 1'bz);
assign p17              = en_spi_0 ? clk_spi_0 : 1'bz;
assign {p51, p50, p15}  = phase_spi_0 ? (en_spi_0 ? (cpol_0 ? ss_n_1_1 : ss_n_0_1): 3'bzzz) : (en_spi_0 ? (cpol_0 ? ss_n_1_0 : ss_n_0_0) : 3'bzzz);

// SPI 1 (SPORT  0)
assign clk_spi_1    = phase_spi_1 ? (cpol_1 ? sclk_1_1 : sclk_0_1) : (cpol_1 ? sclk_1_0 : sclk_0_0);
assign sport_ss_n   = phase_spi_1 ? (cpol_1 ? ss_n_1_1 : ss_n_0_1) : (cpol_1 ? ss_n_1_0 : ss_n_0_0) ;
assign sport_dt0    = phase_spi_1 ? (cpol_1 ? mosi_1_1 : mosi_0_1) : (cpol_1 ? mosi_1_0 : mosi_0_0);
assign p18          = en_spi_1 ? (sport_ss_n[0] & sport_ss_n[2]) : 1'bz;           //rfs
assign p4           = en_spi_1 ? ((sport_ss_n[0] & sport_ss_n[2]) ? 1'bz: clk_spi_1 ) : 1'bz;
assign p10          = en_spi_1 ? (sport_ss_n[0] & sport_ss_n[1]) : 1'bz;           //tfs
assign p29          = en_spi_1 ? ((sport_ss_n[0] & sport_ss_n[1]) ? 1'bz :clk_spi_1 ) : 1'bz;
assign p49          = en_spi_1 ? ((sport_ss_n[0] & sport_ss_n[1]) ? 1'bz :sport_dt0) : 1'bz;

//MISO for both SPI and SPORT
assign miso_0_0         = (spi_active == 3'b100) ? p2 : (sport_active == 3'b100) ? p3 :1'b0;
assign miso_0_1         = (spi_active == 3'b101) ? p2 : (sport_active == 3'b101) ? p3 :1'b0;
assign miso_1_0         = (spi_active == 3'b110) ? p2 : (sport_active == 3'b110) ? p3 :1'b0;
assign miso_1_1         = (spi_active == 3'b111) ? p2 : (sport_active == 3'b111) ? p3 :1'b0;

// other
assign led              = ~led_port;    // inverted since LED shine when output is low
assign p60              = 1'b1;         // reset request not needed
assign adc_data_ready   = {p38,p3, p2};//adc_data_ready[0]= SPI_MISO; adc_data_ready[1]=sport_dr0; adc_data_ready[2]=sport_dr1

endmodule
