/**************************************************************************//**
*   @file   AD5684R.h
*   @brief  Header file of AD5684R Driver for Nios II processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#ifndef _AD5684R_H_
#define _AD5684R_H_
#ifdef AD5684R_GLOBALS
	#define AD5684R_EXT extern volatile
#else
	#define AD5684R_EXT volatile
#endif

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "adi.h"

#if(USE_AD5684R == 1)
/*****************************************************************************/
/* SPI Configuration                                                         */
/*****************************************************************************/
#define AD5684R_SPI		SPI_0_P1_BASE
#define AD5684R_SPI_CFG	(SPI1_ena | SPI1_phase)

/*****************************************************************************/
/* Register description                                                      */
/*****************************************************************************/
// [23:20] Command Bits
#define CMD_NO_OP		(0 << 20)
#define CMD_WR_INPUT	(1 << 20)
#define CMD_UP_DAC		(2 << 20)
#define CMD_WR_UP_DAC	(3 << 20)
#define CMD_PWR_DAC		(4 << 20)
#define CMD_HW_LDAC_MSK	(5 << 20)
#define CMD_SW_RST		(6 << 20)
#define CMD_INT_REF		(7 << 20)
#define CMD_DAISY		(8 << 20)
#define CMD_READBK		(9 << 20)

// [19:16] Address Bits
#define DAC_A			(1 << 16)
#define DAC_B			(1 << 17)
#define DAC_C 			(1 << 18)
#define DAC_D			(1 << 19)
#define DAC_ALL			(15 << 19)

// [18:4] Data Bits
#define DAC_DATA_MSK	0x00FFFF
#define DAC_DATA_OFST	0

/*****************************************************************************/
/* Monitored variables declarations                                          */
/*****************************************************************************/
AD5684R_EXT uCProbeDataUnion AD5684R_Data[21];

#endif

int AD5684R_Init(void);
int AD5684R_UpdateParameters(void);

#endif
